<?php

namespace MoOTP;

use MoOTP\Traits\Instance;
use MoOTP\Helper\MenuItems;
use MoOTP\Handler\MoOTPActionHandlerHandler;
use MoOTP\Handler\LoginRegistrationForm;
use MoOTP\Handler\MoRegistrationHandler;
use MoOTP\Helper\MoDisplayMessages;
use MoOTP\Helper\MoMessages;
use MoOTP\Helper\CountryList;
use MoOTP\Helper\MoUtility;

if(! defined( 'ABSPATH' )) exit;

final class MoOTPInitializer
{

	use Instance;

	private function __construct()
	{
		$this->initialize_values();
		$this->initialize_hooks();
		$this->initialize_helpers();
		$this->initialize_handlers();

	}

	private function initialize_hooks()
	{
		add_action( 'plugins_loaded'				, array( $this, 'mofr_otp_load_textdomain'						 )		  );
		add_action( 'admin_menu'             , array( $this, 'miniorange_customer_validation_menu'));
		add_action( 'admin_enqueue_scripts'	 , array( $this, 'mo_registration_plugin_settings_style'));
		add_action( 'admin_enqueue_scripts'	 , array( $this, 'mo_registration_plugin_settings_script'));
		add_action( 'mo_registration_show_message'	, array( $this, 'mo_show_otp_message'    		 			 ),1   , 2);
		add_action( 'admin_footer'                  , array( $this,	'mofr_feedback_request'));
        add_action( 'wp_enqueue_scripts'		  	, array( $this, 'mo_registration_plugin_frontend_scripts' 	 ),99	  );
		add_action( 'login_enqueue_scripts'		  	, array( $this, 'mo_registration_plugin_frontend_scripts' 	 ),99	  );

	}

	function mofr_feedback_request()
	{
		include MOFLR_DIR . 'controllers/mofrfeedback.php';
	}

	private function initialize_values(){
		if(!get_mofr_option("mofr_firebase_auth_key") && (get_mofr_option("mofr_firebase_auth_key") != "expired")){
				update_mofr_option("mofr_firebase_auth_key",10);
		}
	}

	private function initialize_handlers()
	{
		MoOTPActionHandlerHandler::instance();
        LoginRegistrationForm::instance();		
		MoRegistrationHandler::instance();
		
	}

	 /**
     * Initialize all the helper classes
     */
	private function initialize_helpers()
	{
		MoMessages::instance();    
	}

	function miniorange_customer_validation_menu()
	{
	    MenuItems::instance();
	}

	function mofr_otp_load_textdomain()
	{
		load_plugin_textdomain( 'miniorange-firebase-otp', FALSE, dirname( plugin_basename(__FILE__) ) . '/lang/' );
		do_action('mofr_firebase_otp_add_on_lang_files');
	}

	function mo_registration_plugin_settings_style()
	{
		wp_enqueue_style( 'mofr_customer_validation_admin_settings_style'	 , MOFLR_CSS_URL);
		wp_enqueue_style( 'mofr_customer_validation_inttelinput_style', MOFLR_INTTELINPUT_CSS);
		wp_enqueue_style( 'mofr_customer_validation_firebase_boot_style', MOFLR_BOOT_CSS);
		wp_enqueue_style( 'mofr_main_css', MOFLR_MAIN_CSS);
	}

	function mo_registration_plugin_settings_script()
	{
		$countryVal = [];
		wp_enqueue_script( 'mofr_customer_validation_admin_settings_script', MOFLR_JS_URL , array('jquery'));
	}
    

    function mo_registration_plugin_frontend_scripts()
	{         $countryVal = [];
		if(!get_mofr_option('mo_firebase_dropdown_form_enable')) return;
		//$selector = add_filter( 'mo_phone_dropdown_selector', array($this,'getPhoneNumberSelector'),1,1);
		$selector = apply_filters( 'mofr_phone_dropdown_selector', array() );
		if (MoUtility::is_blank($selector)) return;
		$selector = array_unique($selector); 
		$countriesavail = CountryList::getCountryCodeList();
		$countriesavail = apply_filters('selected_countries',$countriesavail);
        foreach ($countriesavail as $key => $value) {
            array_push($countryVal, $value);
        }
        $defaultCountry = CountryList::getDefaultCountryIsoCode();
        $getIpcountry = apply_filters( 'mofr_get_default_country',$defaultCountry);
		wp_register_script('mofr_customer_validation_inttelinput_script', MOFLR_INTTELINPUT_JS , array('jquery'));
		wp_localize_script('mofr_customer_validation_inttelinput_script', 'moselecteddropdown', array(
			'selecteddropdown' =>  $countryVal,

		));
		wp_enqueue_script('mofr_customer_validation_inttelinput_script');

		wp_enqueue_style( 'mofr_customer_validation_inttelinput_style', MOFLR_INTTELINPUT_CSS);
		wp_register_script('mofr_customer_validation_dropdown_script', MOFLR_DROPDOWN_JS , array('jquery'), MOFLR_VERSION, true);
		wp_localize_script('mofr_customer_validation_dropdown_script', 'modropdownvars', array(
			'selector' =>  json_encode($selector),
			'defaultCountry' => $getIpcountry,
			'onlyCountries' => CountryList::getOnlyCountryList(),
		));
		wp_enqueue_script('mofr_customer_validation_dropdown_script');
	}
    /**
	 * This function runs when mo_registration_show_message hook
	 * is initiated. The hook runs to show a plugin generated
	 * message to the user in the admin dashboard.
	 *
	 * @param $content refers to the message content
	 * @param $type refers to the type of message
	 */
	function mo_show_otp_message($content,$type)
	{
		new MoDisplayMessages($content,$type);
	}

	function  mo_customer_validation_options()
	{
		include MOFLR_DIR . 'controllers/main-controller.php';
	}

}
