<?php

namespace MoOTP\Handler;
if(! defined( 'ABSPATH' )) exit;

use MoOTP\Objects\TabDetails;
use MoOTP\Objects\Tabs;
use MoOTP\Helper\MoConstants;
use MoOTP\Traits\Instance;
use MoOTP\Helper\MoMessages;
use MoOTP\Helper\MoUtility;
use MoOTP\Objects\BaseActionHandler;
use MoOTP\Helper\MocURLOTP;
use MoOTP\Helper\CountryList;

class MoOTPActionHandlerHandler extends BaseActionHandler
{
    use Instance;
	function __construct()
	{
	     parent::__construct();
		$this->_nonce = 'mo_admin_actions';

		add_action( 'admin_init',  array( $this, 'mofr_handle_admin_actions' ) );
	}


	/**
	 * This function hooks into the admin_init wordpress hook. This function
	 * checks the form being posted and routes the data to the correct function
	 * for processing. The 'option' value in the form post is checked to make
	 * the diversion.
	 */
	function mofr_handle_admin_actions()
	{
		$POST = MoUtility::sanitize_post_data($_POST);
		
		if(!isset($POST['option'])) return;
		switch(sanitize_text_field($_POST['option']))
		{
			case "mo_firebase_gateway_deatils_save_option":
				$this->_mo_save_firebase_gateway_details($_POST);
				break;
			case "mofr_check_mo_ln":
                $this->_mofr_check_l();
                break;
            case "mofr_validation_contact_us_query_option":
				$this->_mofr_validation_support_query($_POST);
				break;
			case "mo_firebase_dropdown_form_save_option":
				$this->_save_extra_settings($_POST); 							
				break;
			case "mofr_otp_feedback_option":
			    $this->_mofr_validation_feedback_query();
			    break;
		}
	}

	/**
     * This function processes the support form data before sending it to the server.
     *
     * @param array $postData
    */
	function _mofr_validation_support_query($postData)
	{
	    $email = MoUtility::sanitize_check('query_email',$postData);
	    $query = MoUtility::sanitize_check('query',$postData);
	    $phone = MoUtility::sanitize_check('query_phone',$postData);

		if(!$email || !$query)
		{
			do_action('mo_registration_show_message', MoMessages::showMessage(MoMessages::SUPPORT_FORM_VALUES),'ERROR');
			return;
		}

		$submitted  = MocURLOTP::mofr_submit_contact_us( $email, $phone, $query );

		if(json_last_error() == JSON_ERROR_NONE && $submitted)
		{
			do_action('mo_registration_show_message',MoMessages::showMessage(MoMessages::SUPPORT_FORM_SENT),'SUCCESS');
			return;
		}

		do_action('mo_registration_show_message',MoMessages::showMessage(MoMessages::SUPPORT_FORM_ERROR),'ERROR');
	}

	function _mofr_validation_feedback_query()
    {
        $submitType = sanitize_text_field($_POST['miniorange_fr_feedback_submit']);
        $deactivatingPlugin = strcasecmp(sanitize_text_field($_POST['plugin_deactivated']),"true")==0;
        $type =  !$deactivatingPlugin ? mofr_("[ Plugin Feedback ] : ") : mofr_("[ Plugin Deactivated ]");

        $views=array();
		$deactivationreasons =  $this->mo_feedback_reasons();
		if(isset($_POST['miniorange_fr_feedback_submit'])){
			if(!empty($_POST['reason'])) {
				$data = mofr_sanitize_array($_POST['reason']);
				foreach($data as $value){
					$views[] = $deactivationreasons[$value];
					}
				}
			}
		$feedback = implode(" , ", $views)." , ".sanitize_text_field($_POST['query_feedback']);
        //$feedback = sanitize_text_field($_POST['query_feedback']);
        $feedbackTemplate = file_get_contents(MOFLR_DIR . 'includes/html/feedback.min.html');

        $current_user = wp_get_current_user();
        $email = get_mofr_option("admin_email");
        $feedbackTemplate = str_replace("{{FIRST_NAME}}",$current_user->first_name,$feedbackTemplate);
        $feedbackTemplate = str_replace("{{LAST_NAME}}",$current_user->last_name,$feedbackTemplate);
        $feedbackTemplate = str_replace("{{PLUGIN_TYPE}}",MOFLR_TYPE.":",$feedbackTemplate);
        $feedbackTemplate = str_replace("{{SERVER}}",sanitize_text_field($_SERVER['SERVER_NAME']),$feedbackTemplate);
        $feedbackTemplate = str_replace("{{EMAIL}}",$email,$feedbackTemplate);
        $feedbackTemplate = str_replace("{{PLUGIN}}",MoConstants::PLUGIN_NAME,$feedbackTemplate);
        $feedbackTemplate = str_replace("{{VERSION}}",MOFLR_VERSION,$feedbackTemplate);
        $feedbackTemplate = str_replace("{{TYPE}}",$type,$feedbackTemplate);
        $feedbackTemplate = str_replace("{{FEEDBACK}}",$feedback,$feedbackTemplate);
        
        $notif = MoUtility::send_email_notif(
            $email,
            "Xecurify",
            MoConstants::FEEDBACK_EMAIL,
            "Firebase OTP Verification Plugin Feedback",
            $feedbackTemplate 
        );
        
        if($notif) {

            do_action('mo_registration_show_message',MoMessages::showMessage(MoMessages::FEEDBACK_SENT),'SUCCESS');
        } else {
            do_action('mo_registration_show_message',MoMessages::showMessage(MoMessages::FEEDBACK_ERROR),'ERROR');
        }

        if($deactivatingPlugin) deactivate_plugins( [MOFLR_PLUGIN_NAME]);
    }

	function _mo_save_firebase_gateway_details($data){

		update_mofr_option('gateway_apiKey' ,MoUtility::sanitize_check('mo_firebase_gateway_apiKey',$data));
		update_mofr_option('gateway_authdomain' ,MoUtility::sanitize_check('mo_firebase_gateway_authdomain',$data));
		update_mofr_option('gateway_databaseurl' ,MoUtility::sanitize_check('mo_firebase_gateway_databaseurl',$data));
		update_mofr_option('gateway_projectid' ,MoUtility::sanitize_check('mo_firebase_gateway_projectid',$data));
		update_mofr_option('gateway_storagebucket' ,MoUtility::sanitize_check('mo_firebase_gateway_storagebucket' ,$data));
		update_mofr_option('gateway_messagingsenderid' ,MoUtility::sanitize_check('mo_firebase_gateway_messagingsenderid',$data));
		update_mofr_option('gateway_appid' ,MoUtility::sanitize_check('mo_firebase_gateway_appid',$data));

		$firebase_gateway_details = array(
			'gateway_apiKey'     => MoUtility::sanitize_check('mo_firebase_gateway_apiKey',$data),
			'gateway_authdomain' => MoUtility::sanitize_check('mo_firebase_gateway_authdomain',$data),
			'gateway_databaseurl' => MoUtility::sanitize_check('mo_firebase_gateway_databaseurl',$data),
			'gateway_projectid'    => MoUtility::sanitize_check('mo_firebase_gateway_projectid',$data), 
		    'gateway_storagebucket' => MoUtility::sanitize_check('mo_firebase_gateway_storagebucket',$data), 
		    'gateway_messagingsenderid' => MoUtility::sanitize_check('mo_firebase_gateway_messagingsenderid',$data), 
		    'gateway_appid' => MoUtility::sanitize_check('mo_firebase_gateway_appid',$data) 
		);

		update_mofr_option("firebase_gateway_details",serialize($firebase_gateway_details));
		
	}

	function mo_feedback_reasons()
	{
		$deactivationreasons = array( 
			"unable_to_setup_plugin"    => "Unable to setup plugin" ,
			"not_the_feture_i_wanted"   => "Features I wanted are missing",
			"lack_of_documentation"     => "Lack of documentation", 
			"cost_is_too_high"          => "Cost is too high" , 
			"found_a_better_plugin"     => "Found a better plugin",
		);

	   return $deactivationreasons;
	}
    
    function _save_extra_settings($posted)
	{
		//$this->is_valid_request();

		delete_site_option('default_country_code');
		$defaultCountry = isset($posted['default_country_code']) ? sanitize_text_field($posted['default_country_code']) : '';

		update_mofr_option('default_country'            ,maybe_serialize(CountryList::$countries[$defaultCountry]));
		update_mofr_option('mo_firebase_dropdown_form_enable'       ,MoUtility::sanitize_check('mo_firebase_dropdown_form_enable',$_POST));		
	}

    function _mofr_check_l()
    {
        $this->is_valid_request();
       MoUtility::mofr_handle_mo_check_ln(true,
            get_mofr_option('admin_customer_key'),
            get_mofr_option('admin_api_key')
        );
    }
    
}