<?php

namespace MoOTP\Handler\Forms;

use MoOTP\Helper\MoPHPSessions;
use MoOTP\Traits\Instance;
use MoOTP\Helper\MoUtility;
use MoOTP\Objects\FormHandler;
use MoOTP\Objects\IFormHandler;
use \WP_Error;
use \WP_User;

class LoginForm extends FormHandler implements IFormHandler
{
    use Instance;

    /**
     * Login form css
     * @var string
     */
    protected $_loginFormCss;

    protected $_nonce             = 'frb_form_nonce';
    protected $_generateOTPAction = 'miniorange_frb_login_generate_otp';

    protected function __construct()
    {
        $this->_isFormEnabled = get_mofr_option('login_form_enable');
        $this->_isDropdownEnabled = get_mofr_option('mo_firebase_dropdown_form_enable');
        $this->_phoneKey = 'mofr_loginwithotp_phone';
        $this->_phoneFormId = [
            '.class_'.$this->_phoneKey,
            'input[name='.$this->_phoneKey.']'
            ];
        $this->getUserPhone = true;
        add_action(  'init', array($this,'handle_mofirebase_form') ,1 );
        parent::__construct();
    }

    function handle_mofirebase_form(){

        add_shortcode('mo_wp_login_form',array($this,'show_mo_wp_login_form') );

        add_action('wp_enqueue_scripts',array($this, 'miniorange_login_mofirebaselogin_script'));
        add_action('wp_enqueue_scripts',array($this, 'miniorange_reset_password_script'));

        add_action("wp_ajax_mofr_login_with_otp_sendotp", [$this,'_send_otp']);
        add_action("wp_ajax_nopriv_mofr_login_with_otp_sendotp", [$this,'_send_otp']);

        add_action("wp_ajax_mofr_reset_pass", [$this,'_reset_pass_otp']);
        add_action("wp_ajax_nopriv_mofr_reset_pass", [$this,'_reset_pass_otp']);

        add_action("wp_ajax_mofr_login_with_email_pass", [$this,'mofr_login_with_email_pass']);
        add_action("wp_ajax_nopriv_mofr_login_with_email_pass", [$this,'mofr_login_with_email_pass']);

        add_action( 'wp_ajax_mofr_login_with_otp_verifyotp',[$this,'mofr_validate_otp_token']);
        add_action( 'wp_ajax_nopriv_mofr_login_with_otp_verifyotp', [$this,'mofr_validate_otp_token']);

        add_action( 'wp_ajax_mofr_reset_pass_verifyotp',[$this,'mofr_validate_reset_pass_otp_token']);
        add_action( 'wp_ajax_nopriv_mofr_reset_pass_verifyotp', [$this,'mofr_validate_reset_pass_otp_token']);

        add_action("wp_ajax_mofr_reset_new_pass", [$this,'_reset_password']);
        add_action("wp_ajax_nopriv_mofr_reset_new_pass", [$this,'_reset_password']);

        $POST = MoUtility::sanitize_post_data($_POST);
        
        if (isset($POST['option']) && $POST['option']=="mo_firebase_login_form_save_option" && isset($POST['nonce_mo_firebase_login_form_save_option']) && wp_verify_nonce($POST['nonce_mo_firebase_login_form_save_option'], "mo_nonce_mo_firebase_form_save_option" ) ) {
            $this->handle_mofirebase_form_options();
        }

    }

    function mofr_login_with_email_pass(){
        
        $nonce = sanitize_text_field($_POST['user_nonce_login_with_emailpass']);

        if ( ! wp_verify_nonce( $nonce, $this->_nonce ) ){
            wp_send_json( MoUtility::create_json("Invalid Operation. Please Try Again !","error") );
        }

        $user_data = sanitize_text_field($_POST['user_data']);
        $user_pass = sanitize_text_field($_POST['user_pass']);

        if(is_email($user_data)){
            $user = get_user_by( 'email', $user_data );
            $username = $user->user_login;
        }
        else{
            $username = $user_data;
        }

        $user = wp_authenticate_username_password('', $username, $user_pass);
        
        if (is_wp_error($user)) {
            wp_send_json(MoUtility::create_json("Invalid Username or Password !","error"));
        }else{

            wp_set_auth_cookie($user->data->ID);
            wp_send_json(MoUtility::create_json("validation successful","success"));
            exit();
        }
    }

    function _send_otp(){

        $token = sanitize_text_field($_POST["recaptcha_token"]);
        $phone = sanitize_text_field($_POST["user_phone"]);
        $user = $this->getUserFromPhoneNumber($phone);
        
        if(!$user){
            wp_send_json( MoUtility::create_json("Please enter a registered Phone Number","error") );    
        }
        else{
            MoPHPSessions::add_session_var('mofr_login_phone_verified',$phone);
        }
        
    }

    function _reset_pass_otp(){

        $nonce = sanitize_text_field($_POST['user_nonce_reset_pass']);

        if ( ! wp_verify_nonce( $nonce, $this->_nonce ) ){
            wp_send_json( MoUtility::create_json("Invalid Operation. Please Try Again !","error") );
        }

        $token = sanitize_text_field($_POST["recaptcha_token"]);

        $email = sanitize_text_field($_POST["user_email"]);
        $user = get_user_by_email( $email);
        $userId = $user->ID;
        $phone = get_user_meta( $userId, "billing_phone");
        $user_phone = $phone[0];

    
        if (MoUtility::is_blank($user_phone)) {
            wp_send_json( MoUtility::create_json("Please enter a registered email address!","error") );
        }
        else{
            MoPHPSessions::add_session_var('mofr_reset_pass_email_verified',$email);
            wp_send_json( MoUtility::create_json($user_phone,"success") );
        }
        
    }

    public function mofr_validate_otp_token()
    {
        if($_POST['mofrlogin_text'] == "OTP verified successfully!!!"){            
            $otpToken = sanitize_text_field($_POST['otp_token']);
            $user = $this->getUserFromPhoneNumber(sanitize_text_field($_POST['user_phone']));
            $phone= sanitize_text_field($_POST['user_phone']);
            if(MoPHPSessions::get_session_var('mofr_login_phone_verified')!=$phone){
                wp_send_json( MoUtility::create_json("The phone number OTP was sent to and the phone number in 
                                                        contact submission do not match.","error") );
            }
            else{
                wp_set_auth_cookie($user->ID);
                wp_send_json( MoUtility::create_json("otp_validated_successfully","success") );                 
            }
        }
    }

    public function mofr_validate_reset_pass_otp_token()
    {
        if($_POST['moResetPassText'] == "OTP verified successfully!!!"){            
            $email= sanitize_text_field($_POST['user_email']);
            if(MoPHPSessions::get_session_var('mofr_reset_pass_email_verified')!=$email){
                wp_send_json( MoUtility::create_json("The Email OTP was sent to and the Email in 
                                                        contact submission do not match.","error") );
            }
            else{
                wp_send_json( MoUtility::create_json("otp_validated_successfully","success") );                 
            }
        }
    }

    public function getUserByEmail($email){
        $email = sanitize_text_field($_POST["user_email"]);
        $user = get_user_by_email( $email);
        $userId = $user->ID;
        $phone = get_user_meta( $userId, "billing_phone");
        $user_phone = $phone[0];

        return $user_phone;
    }

    function containsSpecialChars($str) {
        $specialChars = preg_match('[@_!#$%^&*()<>?/|}{~:]', $str);
        return $specialChars;
    }

    function _reset_password(){
        global $wpdb;

        $user_data = sanitize_text_field($_POST['user_email']);

        $user = get_user_by("email",$user_data); 
        $user_id =$user->ID;


        $newPass = sanitize_text_field($_POST["new_pass"]);

        $confirmPass = sanitize_text_field($_POST["confirm_pass"]);

        if($newPass != $confirmPass){
            wp_send_json(MoUtility::create_json(
                           "Set Password field and Confirm password fields are not same.", "error"
                    ));
        }
        else if( strlen($newPass) < 6 && !$this->containsSpecialChars($newPass)){
            wp_send_json(MoUtility::create_json(
                           "Password must contain atleast 6 characters and 1 special character.", "error"
                    ));
        }
        else{
            if (isset($newPass) ) {
                wp_set_password( $newPass,  $user->ID );
            }
            wp_send_json(MoUtility::create_json(
                           "Password updated succesfully!", "success"
                    ));
        }
    }

     /**
     * This functions fetches the user associated with a phone number
     *
     * @param string $username  the user's username
     * @return bool|WP_User
     */
    function getUserFromPhoneNumber($username)
    {
        global $wpdb;
        
        $sql = $wpdb->prepare( "SELECT `user_id` FROM `{$wpdb->prefix}usermeta` WHERE `meta_key` = %s AND `meta_value` = %s",array('billing_phone',$username) );
        
        $results = $wpdb->get_row($sql);
        return !MoUtility::is_blank($results) ? get_userdata($results->user_id) : false;
    }

    public function miniorange_login_mofirebaselogin_script(){

        wp_register_script( 'mofrloginform',MOFLR_URL.'includes/js/mofrloginform.js',array('jquery'));
        wp_localize_script( 'mofrloginform', 'mofrloginform', array(
            'siteURL'       => wp_ajax_fr_url(),
            'nonce'         => wp_create_nonce($this->_nonce),
            'generateURL'   => $this->_generateOTPAction,
            'redirecturl'   => get_page_link(get_mofr_option('login_form_redirecturl')),
        ));
        wp_enqueue_script( 'mofrloginform' );
    }

    public function miniorange_reset_password_script(){

        wp_register_script( 'mofrresetpass',MOFLR_URL.'includes/js/mofr_reset_pass.js',array('jquery'));
        wp_localize_script( 'mofrresetpass', 'mofrresetpass', array(
            'siteURL'       => wp_ajax_fr_url(),
            'nonce'         => wp_create_nonce($this->_nonce),
            'generateURL'   => $this->_generateOTPAction,
            'user_phone'    => $this->getUserPhone,
            'redirecturl'   => get_page_link(get_mofr_option('login_form_redirecturl')),
        ));
        wp_enqueue_script( 'mofrresetpass' );
    }

    public function show_mo_wp_login_form()
    {     
        if(is_user_logged_in()){
            return;
        }
        else{
            $customCSS = get_mofr_option('login_form_css');
            include MOFLR_DIR . 'views/shortcodes/mo-user-login-form-shortcode.php';
        }

    }

    public function getPhoneNumberSelector($selector)
    {

        if($this->_isFormEnabled && $this->_isDropdownEnabled) {
            $selector = array_merge($selector,$this->_phoneFormId);
        }
        return $selector;
    }

    function handle_mofirebase_form_options()
    {
        $this->_isFormEnabled = $this->sanitize_form_post('login_form_enable');
        $this->_loginFormCss  = $this->sanitize_form_post('login_form_css');
        $this->_redirecturl   = isset( $post_id) ? $post_id: "";
        $this->_isDropdownEnabled = $this->sanitize_form_post('dropdown_form_enable');

        update_mofr_option('login_form_enable',$this->_isFormEnabled);
        update_mofr_option('login_form_css',$this->_loginFormCss);
        update_mofr_option('login_form_redirecturl',$this->_redirecturl);
        
        
    }
}